use super::BirdCard;
use pyo3::prelude::*;

#[derive(Debug, Clone, Copy, PartialEq, Eq, PartialOrd, Ord, Hash)]
#[pyclass(eq, eq_int)]
pub enum BeakDirection {
  Left = 0,
  Right = 1,
  None = 2,
}

impl BirdCard {
  pub fn beak_direction(&self) -> BeakDirection {
    match self {
      Self::AbbottsBooby => BeakDirection::Left,
      Self::AcornWoodpecker => BeakDirection::Left,
      Self::AmericanAvocet => BeakDirection::Left,
      Self::AmericanBittern => BeakDirection::Right,
      Self::AmericanCoot => BeakDirection::Right,
      Self::AmericanCrow => BeakDirection::Right,
      Self::AmericanGoldfinch => BeakDirection::Right,
      Self::AmericanKestrel => BeakDirection::Left,
      Self::AmericanOystercatcher => BeakDirection::Right,
      Self::AmericanRedstart => BeakDirection::Right,
      Self::AmericanRobin => BeakDirection::Right,
      Self::AmericanWhitePelican => BeakDirection::Right,
      Self::AmericanWoodcock => BeakDirection::Left,
      Self::Anhinga => BeakDirection::Left,
      Self::AnnasHummingbird => BeakDirection::Left,
      Self::AshThroatedFlycatcher => BeakDirection::Right,
      Self::AsianEmeraldDove => BeakDirection::Left,
      Self::AsianKoel => BeakDirection::Right,
      Self::AtlanticPuffin => BeakDirection::Right,
      Self::AudouinsGull => BeakDirection::Right,
      Self::AustralasianPipit => BeakDirection::Right,
      Self::AustralasianShoveler => BeakDirection::Left,
      Self::AustralianIbis => BeakDirection::Right,
      Self::AustralianMagpie => BeakDirection::Right,
      Self::AustralianOwletNightjar => BeakDirection::None,
      Self::AustralianRaven => BeakDirection::Right,
      Self::AustralianReedWarbler => BeakDirection::Left,
      Self::AustralianShelduck => BeakDirection::Left,
      Self::AustralianZebraFinch => BeakDirection::Right,
      Self::AzureTit => BeakDirection::Right,
      Self::BairdsSparrow => BeakDirection::Right,
      Self::BaldEagle => BeakDirection::Left,
      Self::BaltimoreOriole => BeakDirection::Left,
      Self::BarnOwl => BeakDirection::None,
      Self::BarnSwallow => BeakDirection::Right,
      Self::BarredOwl => BeakDirection::None,
      Self::BarrowsGoldeneye => BeakDirection::Left,
      Self::BayaWeaver => BeakDirection::Right,
      Self::BeardedReedling => BeakDirection::None,
      Self::BellsVireo => BeakDirection::Right,
      Self::BeltedKingfisher => BeakDirection::Left,
      Self::BewicksWren => BeakDirection::Right,
      Self::BlackDrongo => BeakDirection::Left,
      Self::BlackNoddy => BeakDirection::Left,
      Self::BlackRedstart => BeakDirection::Right,
      Self::BlackSkimmer => BeakDirection::Left,
      Self::BlackStork => BeakDirection::Left,
      Self::BlackSwan => BeakDirection::Right,
      Self::BlackTern => BeakDirection::Right,
      Self::BlackVulture => BeakDirection::Right,
      Self::BlackWoodpecker => BeakDirection::Right,
      Self::BlackBelliedWhistlingDuck => BeakDirection::Right,
      Self::BlackBilledMagpie => BeakDirection::Right,
      Self::BlackChinnedHummingbird => BeakDirection::Right,
      Self::BlackCrownedNightHeron => BeakDirection::Right,
      Self::BlackHeadedGull => BeakDirection::Left,
      Self::BlackNapedOriole => BeakDirection::Right,
      Self::BlackNeckedStilt => BeakDirection::Right,
      Self::BlackShoulderedKite => BeakDirection::Left,
      Self::BlackTailedGodwit => BeakDirection::Left,
      Self::BlackThroatedDiver => BeakDirection::Left,
      Self::BlueGrosbeak => BeakDirection::Right,
      Self::BlueJay => BeakDirection::Right,
      Self::BlueRockThrush => BeakDirection::Right,
      Self::BlueGrayGnatcatcher => BeakDirection::Left,
      Self::BlueWingedWarbler => BeakDirection::Right,
      Self::Bluethroat => BeakDirection::Right,
      Self::BlythsHornbill => BeakDirection::Right,
      Self::Bobolink => BeakDirection::Right,
      Self::BonellisEagle => BeakDirection::Left,
      Self::BrahminyKite => BeakDirection::Left,
      Self::Brambling => BeakDirection::Right,
      Self::Brant => BeakDirection::Right,
      Self::BrewersBlackbird => BeakDirection::Left,
      Self::BroadWingedHawk => BeakDirection::Right,
      Self::Brolga => BeakDirection::Right,
      Self::BronzedCowbird => BeakDirection::Right,
      Self::BrownFalcon => BeakDirection::Right,
      Self::BrownPelican => BeakDirection::Right,
      Self::BrownShrike => BeakDirection::Left,
      Self::BrownHeadedCowbird => BeakDirection::Left,
      Self::Budgerigar => BeakDirection::Right,
      Self::Bullfinch => BeakDirection::Right,
      Self::BurrowingOwl => BeakDirection::None,
      Self::Bushtit => BeakDirection::Left,
      Self::CaliforniaCondor => BeakDirection::None,
      Self::CaliforniaQuail => BeakDirection::Left,
      Self::CanadaGoose => BeakDirection::Right,
      Self::Canvasback => BeakDirection::Right,
      Self::CarolinaChickadee => BeakDirection::Left,
      Self::CarolinaWren => BeakDirection::Right,
      Self::CarrionCrow => BeakDirection::Right,
      Self::CassinsFinch => BeakDirection::Right,
      Self::CassinsSparrow => BeakDirection::Left,
      Self::CedarWaxwing => BeakDirection::Right,
      Self::CeruleanWarbler => BeakDirection::Left,
      Self::CettisWarbler => BeakDirection::Left,
      Self::ChestnutCollaredLongspur => BeakDirection::Left,
      Self::ChihuahuanRaven => BeakDirection::Right,
      Self::ChimneySwift => BeakDirection::Left,
      Self::ChippingSparrow => BeakDirection::Right,
      Self::ClarksGrebe => BeakDirection::Right,
      Self::ClarksNutcracker => BeakDirection::Right,
      Self::CoalTit => BeakDirection::Right,
      Self::Cockatiel => BeakDirection::Right,
      Self::CommonBlackbird => BeakDirection::Right,
      Self::CommonBuzzard => BeakDirection::Left,
      Self::CommonChaffinch => BeakDirection::Right,
      Self::CommonChiffchaff => BeakDirection::Right,
      Self::CommonCuckoo => BeakDirection::Right,
      Self::CommonGoldeneye => BeakDirection::Left,
      Self::CommonGrackle => BeakDirection::Left,
      Self::CommonGreenMagpie => BeakDirection::Right,
      Self::CommonIora => BeakDirection::Right,
      Self::CommonKingfisher => BeakDirection::Left,
      Self::CommonLittleBittern => BeakDirection::Right,
      Self::CommonLoon => BeakDirection::Left,
      Self::CommonMerganser => BeakDirection::Right,
      Self::CommonMoorhen => BeakDirection::Left,
      Self::CommonMyna => BeakDirection::Right,
      Self::CommonNighthawk => BeakDirection::Right,
      Self::CommonNightingale => BeakDirection::Right,
      Self::CommonRaven => BeakDirection::Left,
      Self::CommonSandpiper => BeakDirection::Right,
      Self::CommonStarling => BeakDirection::Right,
      Self::CommonSwift => BeakDirection::Right,
      Self::CommonTailorbird => BeakDirection::Left,
      Self::CommonTeal => BeakDirection::Left,
      Self::CommonYellowthroat => BeakDirection::Left,
      Self::CoopersHawk => BeakDirection::Left,
      Self::CoppersmithBarbet => BeakDirection::Right,
      Self::CorsicanNuthatch => BeakDirection::Right,
      Self::CountRaggisBirdofParadise => BeakDirection::Left,
      Self::CrestedIbis => BeakDirection::Left,
      Self::CrestedLark => BeakDirection::Left,
      Self::CrestedPigeon => BeakDirection::Right,
      Self::CrimsonChat => BeakDirection::Right,
      Self::DarkEyedJunco => BeakDirection::Left,
      Self::DesertFinch => BeakDirection::Right,
      Self::DesertWheatear => BeakDirection::Left,
      Self::Dickcissel => BeakDirection::Right,
      Self::DoubleCrestedCormorant => BeakDirection::Right,
      Self::DownyWoodpecker => BeakDirection::Left,
      Self::Dunnock => BeakDirection::Right,
      Self::EasternBluebird => BeakDirection::Right,
      Self::EasternImperialEagle => BeakDirection::Left,
      Self::EasternKingbird => BeakDirection::Right,
      Self::EasternPhoebe => BeakDirection::Right,
      Self::EasternRosella => BeakDirection::Right,
      Self::EasternScreechOwl => BeakDirection::None,
      Self::EasternWhipbird => BeakDirection::Left,
      Self::EleonorasFalcon => BeakDirection::Left,
      Self::Emu => BeakDirection::Right,
      Self::EurasianCollaredDove => BeakDirection::Right,
      Self::EurasianCoot => BeakDirection::Right,
      Self::EurasianEagleOwl => BeakDirection::None,
      Self::EurasianGoldenOriole => BeakDirection::Left,
      Self::EurasianGreenWoodpecker => BeakDirection::Right,
      Self::EurasianHobby => BeakDirection::Left,
      Self::EurasianHoopoe => BeakDirection::Right,
      Self::EurasianJay => BeakDirection::Left,
      Self::EurasianKestrel => BeakDirection::Right,
      Self::EurasianMagpie => BeakDirection::Right,
      Self::EurasianMarshHarrier => BeakDirection::Left,
      Self::EurasianNutcracker => BeakDirection::Left,
      Self::EurasianNuthatch => BeakDirection::Left,
      Self::EurasianSparrowhawk => BeakDirection::Right,
      Self::EurasianTreeSparrow => BeakDirection::Right,
      Self::EurasianTreecreeper => BeakDirection::Left,
      Self::EuropeanBeeEater => BeakDirection::Right,
      Self::EuropeanGoldfinch => BeakDirection::Right,
      Self::EuropeanHoneyBuzzard => BeakDirection::Right,
      Self::EuropeanRobin => BeakDirection::Left,
      Self::EuropeanRoller => BeakDirection::Right,
      Self::EuropeanTurtleDove => BeakDirection::Right,
      Self::FerruginousHawk => BeakDirection::Right,
      Self::FireFrontedSerin => BeakDirection::Right,
      Self::FishCrow => BeakDirection::Right,
      Self::ForestOwlet => BeakDirection::None,
      Self::ForstersTern => BeakDirection::Left,
      Self::FranklinsGull => BeakDirection::Right,
      Self::Galah => BeakDirection::Right,
      Self::Goldcrest => BeakDirection::Right,
      Self::GoldenEagle => BeakDirection::Right,
      Self::GoldenPheasant => BeakDirection::Right,
      Self::GoldenHeadedCisticola => BeakDirection::Left,
      Self::GouldsFinch => BeakDirection::Right,
      Self::GracefulPrinia => BeakDirection::Right,
      Self::Grandala => BeakDirection::Right,
      Self::GrasshopperSparrow => BeakDirection::Left,
      Self::GrayCatbird => BeakDirection::Left,
      Self::GrayWagtail => BeakDirection::Right,
      Self::GreatBlueHeron => BeakDirection::Right,
      Self::GreatCormorant => BeakDirection::Right,
      Self::GreatCrestedFlycatcher => BeakDirection::Right,
      Self::GreatCrestedGrebe => BeakDirection::Left,
      Self::GreatEgret => BeakDirection::Right,
      Self::GreatHornbill => BeakDirection::Right,
      Self::GreatHornedOwl => BeakDirection::Right,
      Self::GreatIndianBustard => BeakDirection::Right,
      Self::GreatSpottedWoodpecker => BeakDirection::Right,
      Self::GreatTit => BeakDirection::Left,
      Self::GreaterAdjutant => BeakDirection::Right,
      Self::GreaterFlamingo => BeakDirection::Right,
      Self::GreaterPrairieChicken => BeakDirection::Right,
      Self::GreaterRoadrunner => BeakDirection::Left,
      Self::GreenBeeEater => BeakDirection::Right,
      Self::GreenHeron => BeakDirection::Right,
      Self::GreenPheasant => BeakDirection::Left,
      Self::GreenPygmyGoose => BeakDirection::Left,
      Self::GreyButcherbird => BeakDirection::Right,
      Self::GreyHeron => BeakDirection::Right,
      Self::GreyShrikethrush => BeakDirection::Left,
      Self::GreyTeal => BeakDirection::Left,
      Self::GreyWarbler => BeakDirection::Right,
      Self::GreyHeadedMannikin => BeakDirection::Right,
      Self::GreylagGoose => BeakDirection::Left,
      Self::GriffonVulture => BeakDirection::Left,
      Self::Hawfinch => BeakDirection::Left,
      Self::HermitThrush => BeakDirection::Right,
      Self::HimalayanMonal => BeakDirection::Right,
      Self::HoodedCrow => BeakDirection::Left,
      Self::HoodedMerganser => BeakDirection::Left,
      Self::HoodedWarbler => BeakDirection::Right,
      Self::HornedLark => BeakDirection::None,
      Self::HorsfieldsBronzeCuckoo => BeakDirection::Left,
      Self::HorsfieldsBushlark => BeakDirection::Left,
      Self::HouseCrow => BeakDirection::Left,
      Self::HouseFinch => BeakDirection::Left,
      Self::HouseSparrow => BeakDirection::Right,
      Self::HouseWren => BeakDirection::Right,
      Self::Ibisbill => BeakDirection::Left,
      Self::IncaDove => BeakDirection::Left,
      Self::IndianPeafowl => BeakDirection::Left,
      Self::IndianVulture => BeakDirection::Left,
      Self::IndigoBunting => BeakDirection::Left,
      Self::JuniperTitmouse => BeakDirection::Left,
      Self::Kea => BeakDirection::Left,
      Self::KelpGull => BeakDirection::Right,
      Self::Kereru => BeakDirection::Left,
      Self::Killdeer => BeakDirection::Left,
      Self::KingRail => BeakDirection::Left,
      Self::Korimako => BeakDirection::Left,
      Self::Kakapo => BeakDirection::None,
      Self::LargeBilledCrow => BeakDirection::Right,
      Self::LaughingKookaburra => BeakDirection::Left,
      Self::LazuliBunting => BeakDirection::Right,
      Self::LesserFrigatebird => BeakDirection::Right,
      Self::LesserWhitethroat => BeakDirection::Left,
      Self::LewinsHoneyeater => BeakDirection::Right,
      Self::LincolnsSparrow => BeakDirection::Right,
      Self::LittleBustard => BeakDirection::Left,
      Self::LittleEgret => BeakDirection::Right,
      Self::LittleGrebe => BeakDirection::Left,
      Self::LittleOwl => BeakDirection::None,
      Self::LittlePenguin => BeakDirection::Right,
      Self::LittlePiedCormorant => BeakDirection::Left,
      Self::LittleRingedPlover => BeakDirection::Right,
      Self::LoggerheadShrike => BeakDirection::Left,
      Self::LongTailedTit => BeakDirection::Right,
      Self::MagpieLark => BeakDirection::Right,
      Self::MajorMitchellsCockatoo => BeakDirection::Left,
      Self::Mallard => BeakDirection::Right,
      Self::Malleefowl => BeakDirection::Left,
      Self::MandarinDuck => BeakDirection::Left,
      Self::ManedDuck => BeakDirection::Left,
      Self::ManyColoredFruitDove => BeakDirection::Left,
      Self::MaskedLapwing => BeakDirection::Right,
      Self::MississippiKite => BeakDirection::Left,
      Self::Mistletoebird => BeakDirection::Left,
      Self::MoltonisWarbler => BeakDirection::Right,
      Self::MontagusHarrier => BeakDirection::Right,
      Self::MountainBluebird => BeakDirection::Right,
      Self::MountainChickadee => BeakDirection::Left,
      Self::MourningDove => BeakDirection::Right,
      Self::MuskDuck => BeakDirection::Left,
      Self::MuteSwan => BeakDirection::Left,
      Self::NewHollandHoneyeater => BeakDirection::Left,
      Self::NoisyMiner => BeakDirection::Left,
      Self::NorthIslandBrownKiwi => BeakDirection::Left,
      Self::NorthernBobwhite => BeakDirection::Left,
      Self::NorthernCardinal => BeakDirection::Right,
      Self::NorthernFlicker => BeakDirection::Right,
      Self::NorthernGannet => BeakDirection::Left,
      Self::NorthernGoshawk => BeakDirection::Right,
      Self::NorthernHarrier => BeakDirection::Right,
      Self::NorthernMockingbird => BeakDirection::Left,
      Self::NorthernShoveler => BeakDirection::Left,
      Self::OliveBackedSunbird => BeakDirection::Left,
      Self::OrangeFootedScrubfowl => BeakDirection::Left,
      Self::OrientalBayOwl => BeakDirection::None,
      Self::OrientalMagpieRobin => BeakDirection::Left,
      Self::Osprey => BeakDirection::Left,
      Self::PacificBlackDuck => BeakDirection::Left,
      Self::PaintedBunting => BeakDirection::Right,
      Self::PaintedWhitestart => BeakDirection::Right,
      Self::ParrotCrossbill => BeakDirection::Left,
      Self::PeacefulDove => BeakDirection::Left,
      Self::PeregrineFalcon => BeakDirection::Left,
      Self::PesquetsParrot => BeakDirection::Left,
      Self::PheasantCoucal => BeakDirection::Left,
      Self::PhilippineEagle => BeakDirection::Left,
      Self::PiedBilledGrebe => BeakDirection::Left,
      Self::PileatedWoodpecker => BeakDirection::Right,
      Self::PineSiskin => BeakDirection::Right,
      Self::PinkEaredDuck => BeakDirection::Right,
      Self::PlainsWanderer => BeakDirection::Left,
      Self::PlumbeousRedstart => BeakDirection::Right,
      Self::PrincessStephaniesAstrapia => BeakDirection::Left,
      Self::ProthonotaryWarbler => BeakDirection::Left,
      Self::PurpleGallinule => BeakDirection::Right,
      Self::PurpleHeron => BeakDirection::Left,
      Self::PurpleMartin => BeakDirection::Left,
      Self::PygmyNuthatch => BeakDirection::Right,
      Self::Pukeko => BeakDirection::Left,
      Self::RainbowLorikeet => BeakDirection::Right,
      Self::RedAvadavat => BeakDirection::Right,
      Self::RedCrossbill => BeakDirection::Left,
      Self::RedJunglefowl => BeakDirection::Left,
      Self::RedKite => BeakDirection::Left,
      Self::RedKnot => BeakDirection::Right,
      Self::RedWattlebird => BeakDirection::Right,
      Self::RedBackedFairywren => BeakDirection::Right,
      Self::RedBackedShrike => BeakDirection::Right,
      Self::RedBelliedWoodpecker => BeakDirection::Right,
      Self::RedBreastedMerganser => BeakDirection::Right,
      Self::RedBreastedNuthatch => BeakDirection::Right,
      Self::RedCappedRobin => BeakDirection::Left,
      Self::RedCockadedWoodpecker => BeakDirection::Right,
      Self::RedCrownedCrane => BeakDirection::Right,
      Self::RedEyedVireo => BeakDirection::Right,
      Self::RedHeadedWoodpecker => BeakDirection::Right,
      Self::RedLeggedPartridge => BeakDirection::Right,
      Self::RedNeckedAvocet => BeakDirection::Right,
      Self::RedShoulderedHawk => BeakDirection::Right,
      Self::RedTailedHawk => BeakDirection::Right,
      Self::RedVentedBulbul => BeakDirection::Right,
      Self::RedWattledLapwing => BeakDirection::Left,
      Self::RedWingedBlackbird => BeakDirection::Left,
      Self::RedWingedParrot => BeakDirection::Right,
      Self::RegentBowerbird => BeakDirection::Right,
      Self::RhinocerosAuklet => BeakDirection::Right,
      Self::RingBilledGull => BeakDirection::Right,
      Self::RockPigeon => BeakDirection::Left,
      Self::Rook => BeakDirection::Left,
      Self::RoseBreastedGrosbeak => BeakDirection::Left,
      Self::RoseRingedParakeet => BeakDirection::Left,
      Self::RoseateSpoonbill => BeakDirection::Right,
      Self::RosyStarling => BeakDirection::Left,
      Self::RoyalSpoonbill => BeakDirection::Right,
      Self::RubyCrownedKinglet => BeakDirection::Right,
      Self::RubyThroatedHummingbird => BeakDirection::Left,
      Self::RuddyDuck => BeakDirection::Right,
      Self::RuddyShelduck => BeakDirection::Right,
      Self::Ruff => BeakDirection::Left,
      Self::RufousNightHeron => BeakDirection::Right,
      Self::RufousOwl => BeakDirection::None,
      Self::RufousBandedHoneyeater => BeakDirection::Left,
      Self::SacredKingfisher => BeakDirection::Right,
      Self::SandhillCrane => BeakDirection::Left,
      Self::SarusCrane => BeakDirection::Right,
      Self::SatyrTragopan => BeakDirection::Left,
      Self::SavannahSparrow => BeakDirection::Right,
      Self::SavisWarbler => BeakDirection::Right,
      Self::SaysPhoebe => BeakDirection::Right,
      Self::ScaledQuail => BeakDirection::Right,
      Self::ScalyBreastedMunia => BeakDirection::Left,
      Self::ScissorTailedFlycatcher => BeakDirection::Right,
      Self::ShortToedTreecreeper => BeakDirection::None,
      Self::Silvereye => BeakDirection::Left,
      Self::SmallMinivet => BeakDirection::Right,
      Self::Smew => BeakDirection::Right,
      Self::SnowBunting => BeakDirection::Left,
      Self::SnowyEgret => BeakDirection::Left,
      Self::SnowyOwl => BeakDirection::None,
      Self::SongSparrow => BeakDirection::Right,
      Self::SouthIslandRobin => BeakDirection::Right,
      Self::SouthernCassowary => BeakDirection::Left,
      Self::SpangledDrongo => BeakDirection::Right,
      Self::SplendidFairywren => BeakDirection::Right,
      Self::SpoonBilledSandpiper => BeakDirection::None,
      Self::SpotlessCrake => BeakDirection::Right,
      Self::SpottedDove => BeakDirection::Right,
      Self::SpottedOwl => BeakDirection::None,
      Self::SpottedSandpiper => BeakDirection::Right,
      Self::SpottedTowhee => BeakDirection::Right,
      Self::SpraguesPipit => BeakDirection::Left,
      Self::SquaccoHeron => BeakDirection::Left,
      Self::SriLankaBlueMagpie => BeakDirection::Left,
      Self::SriLankaFrogmouth => BeakDirection::None,
      Self::StellersJay => BeakDirection::Left,
      Self::StorkBilledKingfisher => BeakDirection::Right,
      Self::StubbleQuail => BeakDirection::Right,
      Self::SulphurCrestedCockatoo => BeakDirection::Left,
      Self::SuperbLyrebird => BeakDirection::Right,
      Self::SwainsonsHawk => BeakDirection::Right,
      Self::TawnyFrogmouth => BeakDirection::Right,
      Self::TheklasLark => BeakDirection::Right,
      Self::TreeSwallow => BeakDirection::Left,
      Self::TrumpeterFinch => BeakDirection::Right,
      Self::TrumpeterSwan => BeakDirection::Right,
      Self::TuftedTitmouse => BeakDirection::Left,
      Self::TurkeyVulture => BeakDirection::Right,
      Self::Twite => BeakDirection::Left,
      Self::Tui => BeakDirection::Left,
      Self::VauxsSwift => BeakDirection::Right,
      Self::VerditerFlycatcher => BeakDirection::Right,
      Self::VioletCuckoo => BeakDirection::Left,
      Self::VioletGreenSwallow => BeakDirection::Right,
      Self::WedgeTailedEagle => BeakDirection::Left,
      Self::WelcomeSwallow => BeakDirection::Right,
      Self::WesternMeadowlark => BeakDirection::Right,
      Self::WesternTanager => BeakDirection::Right,
      Self::WhiteStork => BeakDirection::Right,
      Self::WhiteWagtail => BeakDirection::Left,
      Self::WhiteBackedWoodpecker => BeakDirection::Left,
      Self::WhiteBelliedSeaEagle => BeakDirection::Right,
      Self::WhiteBreastedNuthatch => BeakDirection::Right,
      Self::WhiteBreastedWoodswallow => BeakDirection::Left,
      Self::WhiteBrowedTitWarbler => BeakDirection::Right,
      Self::WhiteCrestedLaughingthrush => BeakDirection::Left,
      Self::WhiteCrownedSparrow => BeakDirection::Right,
      Self::WhiteFacedHeron => BeakDirection::Right,
      Self::WhiteFacedIbis => BeakDirection::Left,
      Self::WhiteHeadedDuck => BeakDirection::Left,
      Self::WhiteThroatedDipper => BeakDirection::Left,
      Self::WhiteThroatedKingfisher => BeakDirection::Right,
      Self::WhiteThroatedSwift => BeakDirection::Left,
      Self::WhoopingCrane => BeakDirection::Right,
      Self::WildTurkey => BeakDirection::Right,
      Self::Willet => BeakDirection::Right,
      Self::WillieWagtail => BeakDirection::Left,
      Self::WillowTit => BeakDirection::Right,
      Self::WilsonsSnipe => BeakDirection::Left,
      Self::WilsonsStormPetrel => BeakDirection::Right,
      Self::WoodDuck => BeakDirection::Right,
      Self::WoodStork => BeakDirection::Left,
      Self::Wrybill => BeakDirection::Left,
      Self::YellowBittern => BeakDirection::Right,
      Self::YellowBelliedSapsucker => BeakDirection::Right,
      Self::YellowBilledCuckoo => BeakDirection::Right,
      Self::YellowBreastedChat => BeakDirection::Right,
      Self::YellowHeadedBlackbird => BeakDirection::Right,
      Self::YellowRumpedWarbler => BeakDirection::Right,
      Self::Yellowhammer => BeakDirection::Left,
      Self::ZebraDove => BeakDirection::Right,
    }
  }
}
